#!/bin/sh
# addpatches.sh
# utility to selectively apply non-mandatory patches to the CLIPS source
#
# $Id: $


# globals and defaults
CLIPS_VERSION=6.23
PATCH_SETS=test,ia64
PATCH_BASEDIR=optpatch
SRC_BASEDIR=clipssrc


# interpret arguments
COMMAND=`basename $0`
for x in $*; do
    case $x in
    --clips-version=*)
        CLIPS_VERSION=`echo $x|cut -d= -f2`
        ;;
    --patchsets=*)
        PATCH_SETS=`echo $x|cut -d= -f2`
        ;;
    *)
        echo "usage: $COMMAND [--clips-version=X.Y] [--patchsets=set1[,set2 ...]]" >&2
        exit 2
        ;;
    esac
done


# decide patchfiles to apply
SET_LIST=`echo $PATCH_SETS|tr "," " "`
PATCH_FILES=""
for x in $SET_LIST; do
    PATCH_FILES="$PATCH_FILES $PATCH_BASEDIR/*-[0-9][0-9].v$CLIPS_VERSION-$x.diff"
done


# apply patches to the correct files
for x in $PATCH_FILES; do
    TARGET=`basename $x|cut -d- -f1`
    [ ! -f $SRC_BASEDIR/$TARGET.ORIG ] && cp $SRC_BASEDIR/$TARGET $SRC_BASEDIR/$TARGET.ORIG
    echo "patching $TARGET using $x"
    patch -p0 $SRC_BASEDIR/$TARGET < $x
done



# end.
